"""
test_wernerBraun2019.py

Compare results from Werner and Braun [2019, JSR]

"""

import unittest
import numpy as np

try:
    from AMAT.planet import Planet
except ModuleNotFoundError:
    raise ModuleNotFoundError("Cannot import Planet from AMAT.planet")

try:
	from AMAT.vehicle import Vehicle
except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Vehicle from AMAT.vehicle")

class WernerBraun2019(unittest.TestCase):

	def test_aerocapture_vehicle(self):

		planet = Planet('EARTH')
		planet.loadAtmosphereModel('atmdata/Earth/earth-gram-avg.dat', 0 , 1 ,2, 3)
		planet.h_skip = 125.0E3

		vehicle=Vehicle('EarthSmallSat', 25.97, 66.4, 0.0, np.pi*0.25**2, 0.0, 0.0563, planet)
		vehicle.setInitialState(125.0,0.0,0.0,9.8,0.0,-5.00,0.0,0.0)
		vehicle.setSolverParams(1E-6)
		vehicle.setDragModulationVehicleParams(66.4,4.72)

		print("Computing aerocapture corridor bounds from Werner and Braun [JSR, 2019]...")
		underShootLimit, exitflag_us = vehicle.findUnderShootLimitD(2400.0, 1.0, -20.0, -2.0, 1E-2, 1760.0)
		overShootLimit , exitflag_os =  vehicle.findOverShootLimitD(2400.0, 1.0, -20.0, -2.0, 1E-2, 1760.0)

		# check exitflag_os, exitflag_us = 1 (indicates AMAT found a solution)
		self.assertEqual(exitflag_os, 1)
		self.assertEqual(exitflag_us, 1)

		self.assertAlmostEqual(overShootLimit,  -4.6496, places=2)
		self.assertAlmostEqual(underShootLimit, -5.1410, places=2)

		# Set target orbit = 180 km x 1760, tolerance = 50 km
		vehicle.setTargetOrbitParams(180.0, 1760.0, 50.0)

		# Set entry phase parameters
		# v_switch_kms = 5.0, lowAlt_km = 50.0,
		# numPoints_lowAlt = 101, hdot_threshold = -200.0 m/s.
		# These are somewhat arbitrary based on experience.
		vehicle.setDragEntryPhaseParams(5.0, 50.0, 101, -200.0)

		# Set vehicle initial state to mid corridor EFPA
		vehicle.setInitialState(125.0,0.0,0.0,9.8,0.0,-4.90,0.0,0.0)

		# Propogate a single vehicle trajectory
		vehicle.propogateGuidedEntryD(1.0,1.0,0.1,2400.0)

		self.assertAlmostEqual(min(vehicle.h_kmc), 124.06, delta=2)
		self.assertAlmostEqual(max(vehicle.acc_net_g_full), 3.37, delta=0.3)
		self.assertAlmostEqual(max(vehicle.q_stag_total_full), 378.55, delta=50)

if __name__ == '__main__':
	unittest.main()




import numpy as np
from mayavi import mlab

from AMAT.approach import Approach
from AMAT.planet import Planet
from AMAT.vehicle import Vehicle
from AMAT.orbiter import Orbiter

probe1 = Approach("URANUS",
				  v_inf_vec_icrf_kms=np.array([-9.62521831, 16.51192666, 7.46493598]),
				  rp=(25559 + 345) * 1e3, psi=np.pi,
				  is_entrySystem=True, h_EI=1000e3)

theta_star_arr_probe1 = np.linspace(-1.8, probe1.theta_star_entry, 101)
pos_vec_bi_arr_probe1 = probe1.pos_vec_bi(theta_star_arr_probe1) / 25559e3

x_arr_probe1 = pos_vec_bi_arr_probe1[0][:]
y_arr_probe1 = pos_vec_bi_arr_probe1[1][:]
z_arr_probe1 = pos_vec_bi_arr_probe1[2][:]

planet = Planet("URANUS")
planet.h_skip = 1000e3
planet.loadAtmosphereModel('../../../atmdata/Uranus/uranus-gram-avg.dat', 0, 1, 2, 3, heightInKmFlag=True)
planet.h_low = 120e3
planet.h_trap = 100e3

vehicle = Vehicle('Titania', 3200.0, 146, 0.24, np.pi * 4.5 ** 2.0, 0.0, 1.125, planet)
vehicle.setInitialState(1000.0, -15.22, 75.55, 29.2877, 88.687, -11.0088, 0.0, 0.0)
vehicle.setSolverParams(1E-6)
vehicle.propogateEntry2(2400.0, 0.1, 180.0)

orbiter = Orbiter(vehicle, 4000.0)
orbiter.compute_probe_targeting_trajectory(178 * np.pi / 180, -90)
orbiter.compute_orbiter_deflection_trajectory(182 * np.pi / 180, 89.38)

u = np.linspace(0, 2 * np.pi, 100)
v = np.linspace(0, np.pi, 100)
x = 1 * np.outer(np.cos(u), np.sin(v))
y = 1 * np.outer(np.sin(u), np.sin(v))
z = 1 * np.outer(np.ones(np.size(u)), np.cos(v))

x1 = 1.040381198513972 * np.outer(np.cos(u), np.sin(v))
y1 = 1.040381198513972 * np.outer(np.sin(u), np.sin(v))
z1 = 1.040381198513972 * np.outer(np.ones(np.size(u)), np.cos(v))

x_ring_1 = 1.4 * np.cos(u)
y_ring_1 = 1.4 * np.sin(u)
z_ring_1 = 0.0 * np.cos(u)

x_ring_2 = 1.6 * np.cos(u)
y_ring_2 = 1.6 * np.sin(u)
z_ring_2 = 0.0 * np.cos(u)

mlab.figure(bgcolor=(0, 0, 0))
s1 = mlab.mesh(x, y, z, color=(0.10, 0.55, 0.35))
s2 = mlab.mesh(x1, y1, z1, color=(0.2, 0.4, 0.4), opacity=0.3)
r1 = mlab.plot3d(x_ring_1, y_ring_1, z_ring_1, color=(1, 1, 1), line_width=1, tube_radius=None)
r2 = mlab.plot3d(x_ring_2, y_ring_2, z_ring_2, color=(1, 1, 1), line_width=1, tube_radius=None)

#p1 = mlab.plot3d(x_arr_probe1, y_arr_probe1, z_arr_probe1, color=(1, 1, 1), line_width=3, tube_radius=None)

#p2 = mlab.plot3d(orbiter.x_coast_arr,
#				 orbiter.y_coast_arr,
#				 orbiter.z_coast_arr, color=(1, 1, 0), line_width=3, tube_radius=None)

p3 = mlab.plot3d(orbiter.x_orbit_arr[0: int(len(orbiter.x_orbit_arr/2))],
				 orbiter.y_orbit_arr[0: int(len(orbiter.x_orbit_arr/2))],
				 orbiter.z_orbit_arr[0: int(len(orbiter.x_orbit_arr/2))], color=(1, 0, 1), line_width=3, tube_radius=None)

p4 = mlab.plot3d(orbiter.x_probe_orbit_arr,
				 orbiter.y_probe_orbit_arr,
				 orbiter.z_probe_orbit_arr, color=(0, 1, 0), line_width=3, tube_radius=None)

#p4 = mlab.plot3d(orbiter.x_orbiter_defl_arr,
#				 orbiter.y_orbiter_defl_arr,
#				 orbiter.z_orbiter_defl_arr, color=(0, 1, 0), line_width=3, tube_radius=None)


mlab.show()



"""
test_orbiter.py

Tests for Orbiter class

"""
import numpy as np

from AMAT.planet import Planet
from AMAT.vehicle import Vehicle


try:
	from AMAT.orbiter import Orbiter
except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Orbiter from AMAT.orbiter")


class TestCoast:

	planet = Planet("URANUS")
	planet.h_skip = 1000e3
	planet.loadAtmosphereModel('atmdata/Uranus/uranus-gram-avg.dat', 0, 1, 2, 3, heightInKmFlag=True)
	planet.h_low = 120e3
	planet.h_trap = 100e3

	vehicle = Vehicle('Titania', 3200.0, 146, 0.24, np.pi * 4.5 ** 2.0, 0.0, 1.125, planet)
	vehicle.setInitialState(1000.0, -15.22, 75.55, 29.2877, 88.687, -11.0088, 0.0, 0.0)
	vehicle.setSolverParams(1E-6)
	vehicle.propogateEntry2(2400.0, 0.1, 180.0)

	orbiter = Orbiter(vehicle, 4000.0)
	orbiter.compute_probe_targeting_trajectory(178 * np.pi / 180, -90)
	orbiter.compute_orbiter_deflection_trajectory(182 * np.pi / 180, 90)


	def test_compute_exit_state_atm_relative(self):

		assert abs((self.orbiter.terminal_r - self.planet.RP) / 1e3 - 1000) < 1.0
		assert abs((self.orbiter.terminal_theta) * 180 / np.pi + 190.81) < 0.1
		assert abs((self.orbiter.terminal_phi) * 180 / np.pi - 63.80) < 0.1
		assert abs((self.orbiter.terminal_v) / 1e3 - 20.41) < 0.1
		assert abs((self.orbiter.terminal_g) * 180 / np.pi - 8.78) < 0.1
		assert abs((self.orbiter.terminal_psi) * 180 / np.pi - 273.297) < 0.1

	def test_compute_exit_state_inertial(self):
		assert abs((self.orbiter.terminal_r_bi - self.planet.RP) / 1e3 - 1000) < 1.0
		assert abs((self.orbiter.terminal_theta_bi) * 180 / np.pi + 190.81) < 0.1
		assert abs((self.orbiter.terminal_phi_bi) * 180 / np.pi - 63.80) < 0.1
		assert abs((self.orbiter.terminal_v_bi) / 1e3 - 20.382) < 0.1
		assert abs((self.orbiter.terminal_g_bi) * 180 / np.pi - 8.8035) < 0.1
		assert abs((self.orbiter.terminal_psi_bi) * 180 / np.pi - 273.297) < 0.1

	def test_compute_theta_star_exit(self):
		assert abs(self.orbiter.theta_star_exit * 180 / np.pi - 18.5214165) < 1e-2


	def test_probe_delivery(self):
		assert abs(self.orbiter.h_periapsis_probe/1e3 - -743.23) < 1e-2

	def test_inertial_entry_speed(self):
		assert abs(self.orbiter.v_bi_mag_probe_entry / 1e3 - 20.3808330) < 1e-6

	def test_inertial_gamma_entry(self):
		assert abs(self.orbiter.gamma_inertial_probe_entry*180/np.pi - -14.482985116) < 1e-6

	def test_atm_relative_entry_speed(self):
		assert abs(self.orbiter.v_mag_probe_entry_atm/1e3 - 20.4058249) < 1e-6

	def test_atm_gamma_entry(self):
		assert abs(self.orbiter.gamma_atm_probe_entry*180/np.pi - -14.14625695788) < 1e-6

	def test_atm_heading_entry(self):
		assert abs(self.orbiter.heading_atm_probe_entry*180/np.pi - 87.162773860) < 1e-6

	def test_probe_entry_state(self):
		assert abs(self.orbiter.h_EI / 1e3 - 1000.0) < 1e-6
		assert abs(round(self.orbiter.longitude_probe_entry_bi * 180 / np.pi, 2) - -10.96) < 1e-2
		assert abs(round(self.orbiter.latitude_probe_entry_bi * 180 / np.pi, 2) - 67.25) < 1e-2
		assert abs(round(self.orbiter.v_mag_probe_entry_atm / 1e3, 4) - 20.4058) < 1e-2
		assert abs(round(self.orbiter.heading_atm_probe_entry * 180 / np.pi, 4) - 87.1628) < 1e-2
		assert abs(round(self.orbiter.gamma_atm_probe_entry * 180 / np.pi, 4) - -14.1463) < 1e-2

	def test_orbiter_deflection_trajectory_periapsis(self):
		assert abs(self.orbiter.h_periapsis_orbiter_defl/1e3 - 4033.662882) < 1e-3










"""
test_approach.py

Tests for Approach class

"""

import numpy as np
from numpy import linalg as LA

try:
	from AMAT.approach import Approach
	from AMAT.orbiter import Orbiter, PropulsiveOrbiter

except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Approach from AMAT.approach")


class Test_Approach_Entry:

	approach = Approach("NEPTUNE",
						v_inf_vec_icrf_kms=np.array([17.78952518, 8.62038536, 3.15801163]),
						rp=(24622+400)*1e3, psi=3*np.pi/2,
						is_entrySystem=True, h_EI=1000e3)

	def test_R1_0(self):

		assert (self.approach.R1(0) == np.array([[1,0,0],
		                                       [0, np.cos(0), np.sin(0)],
		                                       [0, -np.sin(0), np.cos(0)]])).all()

	def test_R1_90(self):

		assert (self.approach.R1(np.pi/2) == np.array([[1, 0, 0],
		                                         [0, np.cos(np.pi/2), np.sin(np.pi/2)],
		                                         [0, -np.sin(np.pi/2), np.cos(np.pi/2)]])).all()

	def test_R2_90(self):
		assert (self.approach.R2(np.pi/2) == np.array([[np.cos(np.pi/2), 0,  -np.sin(np.pi/2)],
		                                                 [0, 1, 0],
		                                                 [np.sin(np.pi/2), 0,  np.cos(np.pi/2)]])).all()

	def test_R3_0(self):

		assert (self.approach.R3(0) == np.array([[np.cos(0), np.sin(0), 0],
		                                        [-np.sin(0), np.cos(0), 0],
		                                        [0, 0, 1]])).all()

	def test_R3_90(self):
		assert (self.approach.R3(np.pi/2) == np.array([[np.cos(np.pi/2), np.sin(np.pi/2), 0],
		                                                [-np.sin(np.pi/2), np.cos(np.pi/2), 0],
		                                                [0, 0, 1]])).all()

	def test_ICRF_to_BI_unit_vec(self):

		delta =  abs(np.linalg.norm(self.approach.ICRF_to_BI(np.array([1, 0, 0])))) - 1
		assert delta < 1e-8

	def test_ICRF_to_BI_123(self):

		delta = abs(np.linalg.norm(self.approach.ICRF_to_BI(np.array([1, 2, 3])))) - np.sqrt(14)
		assert delta < 1e-8


	def test_v_inf_mag(self):
		assert abs(self.approach.v_inf_mag_kms - 20.018) < 1e-3
		assert abs(self.approach.v_inf_mag - 20018) < 1

	def test_a(self):
		assert ((abs(self.approach.a) - abs(-17059283.6903)) / abs(-17059283.6903)) < 1e-6

	def test_e(self):
		assert (abs(self.approach.e - 2.466767) / 2.466767) < 1e-4

	def test_beta(self):
		assert (abs(self.approach.beta - 1.153392) / 1.153392) < 1e-4

	def test_v_inf_bi(self):
		assert abs(self.approach.v_inf_vec_bi_mag_kms - 20.018) < 1e-3
		assert abs(np.arcsin(self.approach.v_inf_vec_bi_unit[2])*180/np.pi - 8.7554787) < 1e-4

	def test_phi(self):
		assert abs(self.approach.phi_1 - 0.07416) < 1e-3
		assert abs(self.approach.phi_2 - self.approach.phi_2_analytic) < 1e-8

	def test_rp_vec(self):
		assert (self.approach.rp_vec_bi_dprime[0] - self.approach.rp_vec_bi_dprime_analytic[0]) < 1e-6
		assert (self.approach.rp_vec_bi_dprime[1] - self.approach.rp_vec_bi_dprime_analytic[1]) < 1e-6
		assert (self.approach.rp_vec_bi_dprime[2] - self.approach.rp_vec_bi_dprime_analytic[2]) < 1e-6

	def test_e_vec_bi_unit(self):
		assert (self.approach.e_vec_bi_unit[0] - self.approach.rp_vec_bi_unit[0]) < 1e-6
		assert (self.approach.e_vec_bi_unit[1] - self.approach.rp_vec_bi_unit[1]) < 1e-6
		assert (self.approach.e_vec_bi_unit[2] - self.approach.rp_vec_bi_unit[2]) < 1e-6

	def test_h_vec_bi_unit(self):
		assert abs(np.arccos(self.approach.h_vec_bi_unit[2])*180/np.pi - 8.7554787) < 1e-4

	def test_N_vec_bi_unit(self):
		assert abs(self.approach.N_vec_bi_unit[0] - 0.07409418) < 1e-6
		assert abs(self.approach.N_vec_bi_unit[1] - -0.99725125) < 1e-6
		assert abs(self.approach.N_vec_bi_unit[2] - 0.) < 1e-6

	def test_i(self):
		assert abs(self.approach.i*180/np.pi - 8.75547878) < 1e-4

	def test_OMEGA(self):
		assert abs(self.approach.OMEGA - 4.78655112) < 1e-6

	def test_omega(self):
		assert abs(self.approach.omega - 0.41740417) < 1e-6

	def test_B_plane(self):
		assert abs(LA.norm(self.approach.R_vec_bi_unit) - 1) < 1e-8
		assert abs(LA.norm(self.approach.S_vec_bi_unit) - 1) < 1e-8
		assert abs(LA.norm(self.approach.T_vec_bi_unit) - 1) < 1e-8
		assert abs(self.approach.T_vec_bi_unit_dprime[1] + 1)  < 1e-8
		assert abs(self.approach.R_vec_bi_unit_dprime[0] - 1)  < 1e-8
		assert np.dot(self.approach.R_vec_bi_unit, self.approach.S_vec_bi_unit) < 1e-8
		assert np.dot(self.approach.R_vec_bi_unit, self.approach.T_vec_bi_unit) < 1e-8
		assert np.dot(self.approach.S_vec_bi_unit, self.approach.T_vec_bi_unit) < 1e-8
		assert np.dot(self.approach.S_vec_bi_unit, self.approach.B_vec_bi_unit) < 1e-8

	def test_theta_star_entry(self):
		assert abs(self.approach.theta_star_entry - -0.25726498) < 1e-6

	def test_r_vec_entry_bi(self):
		delta =  abs(LA.norm(self.approach.r_vec_entry_bi) - self.approach.r_EI)
		assert delta < 1e-4

	def test_v_entry_mag(self):
		assert abs(self.approach.v_entry_inertial_mag - 30567.901209444415) < 1e-6

	def test_gamma_entry_inertial(self):
		assert abs(self.approach.gamma_entry_inertial + 0.18330372) < 1e-6

	def test_v_vec_entry_bi(self):
		assert abs(LA.norm(self.approach.v_vec_entry_bi) - 30567.901209444415) < 1e-6

	def test_latitude_entry(self):
		ans1 = self.approach.latitude_entry_bi
		ans2 = self.approach.latitude_entry_bi_analytic
		assert abs(ans1 - ans2) < 1e-8

	def test_longitude_entry(self):
		assert abs(self.approach.longitude_entry_bi + 1.33832990) < 1e-6

	def test_v_vec_entry_atm(self):
		assert abs(self.approach.v_vec_entry_atm[0] - 24906.113142) < 1e-3
		assert abs(self.approach.v_vec_entry_atm[1] - 11733.340980) < 1e-3
		assert abs(self.approach.v_vec_entry_atm[2] - 4381.2517221) < 1e-3

	def test_v_entry_atm_mag(self):
		assert abs(self.approach.v_entry_atm_mag - 27877.9685250) < 1e-4

	def test_gamma_entry_inertial_check(self):
		ans1 = self.approach.gamma_entry_inertial
		ans2 = self.approach.gamma_entry_inertial_check
		assert abs(ans1-ans2) < 1e-6

	def test_gamma_entry_atm(self):
		assert abs(self.approach.gamma_entry_atm + 0.2012221) < 1e-4

	def test_heading_entry(self):
		assert abs(self.approach.heading_entry_atm*180/np.pi - 14.910607) < 1e-4


class Test_Approach_Orbiter:

	approach = Approach("NEPTUNE",
						v_inf_vec_icrf_kms=np.array([17.78952518, 8.62038536, 3.15801163]),
						rp=(24622+4000)*1e3, psi=3*np.pi/2)



	def test_theta_star_periapsis(self):
		ans1 = self.approach.theta_star_periapsis
		ans2 = self.approach.pos_vec_bi(self.approach.theta_star_periapsis)


class Test_Approach_Orbiter_Mars:

	approach = Approach("MARS", v_inf_vec_icrf_kms=np.array([2.23930484, 1.20086474,-0.73683366]),
						rp=(3389.5+200)*1e3, psi=np.pi)

	orbiter = PropulsiveOrbiter(approach=approach, apoapsis_alt_km=2000)

	def test_theta_star_periapsis(self):
		ans1 = self.approach.r_vec_rp_bi
		ans2 = self.approach.v_vec_rp_bi
		pass

	def test_OI_DV(self):
		ans3 = self.orbiter.DV_OI_mag
		pass


class Test_Approach_Entry_Uranus:

	approach = Approach("URANUS",
						v_inf_vec_icrf_kms=np.array([-9.62521831, 16.51192666,  7.46493598]),
						rp=(25559+400)*1e3, psi=3*np.pi/2,
						is_entrySystem=True, h_EI=1000e3)

	def test_gamma_entry_atm(self):
		pass


class Test_Approach_Entry_Mars:

	approach = Approach("MARS",
						v_inf_vec_icrf_kms=np.array([2.23930484, 1.20086474, -0.73683366]),
						rp=(3389 + 20) * 1e3, psi=3*np.pi/2,
						is_entrySystem=True, h_EI=120e3)

	def test_aebeta(self):
		a = self.approach.a
		e = self.approach.e
		beta = self.approach.beta


	def test_v_inf_bi(self):
		v_inf_vec_bi_kms = self.approach.v_inf_vec_bi_kms
		v_inf_vec_bi = self.approach.v_inf_vec_bi
		v_inf_vec_bi_mag_kms = self.approach.v_inf_vec_bi_mag_kms
		v_inf_vec_bi_unit = self.approach.v_inf_vec_bi_unit
		pass

	def test_phi_1_phi_2(self):
		phi_1 = self.approach.phi_1
		phi_2 = self.approach.phi_2
		phi_2_analytic = self.approach.phi_2_analytic
		pass




from mayavi import mlab
import numpy as np
from tvtk.tools import visual
from AMAT.approach import Approach
from astropy.time import Time
from AMAT.arrival import Arrival

def Arrow_From_A_to_B(x1, y1, z1, x2, y2, z2):
    ar1 = visual.arrow(x=x1, y=y1, z=z1)
    ar1.length_cone = 0.4

    arrow_length = np.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2 + (z2 - z1) ** 2)
    ar1.actor.scale = [arrow_length, arrow_length, arrow_length]
    ar1.pos = ar1.pos / arrow_length
    ar1.axis = [x2 - x1, y2 - y1, z2 - z1]
    return ar1

arrival = Arrival()
arrival.set_vinf_vec_from_lambert_arc(lastFlybyPlanet='EARTH',
									  arrivalPlanet='MARS',
									  lastFlybyDate=Time("2020-07-30 00:00:00", scale='tdb'),
									  arrivalDate=Time("2021-02-18 00:00:00", scale='tdb'),
									  ephem_file='../../../spice-data/de432s.bsp')



probe = Approach("MARS",
                    v_inf_vec_icrf_kms=np.array([ 2.23930484,  1.20086474, -0.73683366]),
                    rp=(3389.5+52)*1e3, psi=3*np.pi/2,
                    is_entrySystem=True, h_EI=120e3)


space = Approach("MARS",
                    v_inf_vec_icrf_kms=np.array([ 2.23930484,  1.20086474, -0.73683366]),
                    rp=(3389.5+250)*1e3, psi=np.pi)

north_pole_bi_vec = probe.ICRF_to_BI(arrival.north_pole)

theta_star_arr_probe = np.linspace(-2, probe.theta_star_entry, 101)
pos_vec_bi_arr_probe = probe.pos_vec_bi(theta_star_arr_probe)/3389.5e3


theta_star_arr_space = np.linspace(-1.85, 0.0, 101)
pos_vec_bi_arr_space = space.pos_vec_bi(theta_star_arr_space)/3389.5e3

x_arr_probe = pos_vec_bi_arr_probe[0][:]
y_arr_probe = pos_vec_bi_arr_probe[1][:]
z_arr_probe = pos_vec_bi_arr_probe[2][:]

x_arr_space = pos_vec_bi_arr_space[0][:]
y_arr_space = pos_vec_bi_arr_space[1][:]
z_arr_space = pos_vec_bi_arr_space[2][:]


u = np.linspace(0, 2 * np.pi, 100)
v = np.linspace(0, np.pi, 100)
x = 1*np.outer(np.cos(u), np.sin(v))
y = 1*np.outer(np.sin(u), np.sin(v))
z = 1*np.outer(np.ones(np.size(u)), np.cos(v))

x1 = 1.040381198513972*np.outer(np.cos(u), np.sin(v))
y1 = 1.040381198513972*np.outer(np.sin(u), np.sin(v))
z1 = 1.040381198513972*np.outer(np.ones(np.size(u)), np.cos(v))


x_ring_1 = 1.1*np.cos(u)
y_ring_1 = 1.1*np.sin(u)
z_ring_1 = 0.0*np.cos(u)

x_ring_2 = 1.2*np.cos(u)
y_ring_2 = 1.2*np.sin(u)
z_ring_2 = 0.0*np.cos(u)

mlab.figure(bgcolor=(0,0,0))
s1 = mlab.mesh(x, y, z, color=(0.7,0.3,0.0))
s2 = mlab.mesh(x1, y1, z1, color=(0.7, 0.3,0.0), opacity=0.3)
r1 = mlab.plot3d(x_ring_1, y_ring_1, z_ring_1, color=(1,1,1), line_width=1, tube_radius=None)
#r2 = mlab.plot3d(x_ring_2, y_ring_2, z_ring_2, color=(1,1,1), line_width=1, tube_radius=None)

p1 = mlab.plot3d(x_arr_probe, y_arr_probe, z_arr_probe, color=(0,1,0), line_width=3, tube_radius=None)
#p2 = mlab.plot3d(x_arr_space, y_arr_space, z_arr_space, color=(0,0,1), line_width=3, tube_radius=None)


mlab.plot3d([0, 1.2 * north_pole_bi_vec[0]],
                   [0, 1.2 * north_pole_bi_vec[1]],
                   [0, 1.2 * north_pole_bi_vec[2]])

mlab.show()



import numpy as np
from mayavi import mlab

from AMAT.approach import Approach
from AMAT.planet import Planet
from AMAT.vehicle import Vehicle
from AMAT.orbiter import Orbiter, PropulsiveOrbiter
from astropy.time import Time
from AMAT.arrival import Arrival

arrival = Arrival()
arrival.set_vinf_vec_from_lambert_arc(lastFlybyPlanet='EARTH',
									  arrivalPlanet='MARS',
									  lastFlybyDate=Time("2020-07-30 00:00:00", scale='tdb'),
									  arrivalDate=Time("2021-02-18 00:00:00", scale='tdb'),
									  ephem_file='../../../spice-data/de432s.bsp')


probe = Approach("MARS",v_inf_vec_icrf_kms=np.array([ 2.23930484,  1.20086474, -0.73683366]),
                        rp=(3389.5+52)*1e3, psi=3*np.pi/2,
                        is_entrySystem=True, h_EI=120e3)

theta_star_arr_probe1 = np.linspace(-1.9, probe.theta_star_entry, 101)
pos_vec_bi_arr_probe1 = probe.pos_vec_bi(theta_star_arr_probe1) / 3389.5e3


space = Approach("MARS",
                    v_inf_vec_icrf_kms=np.array([ 2.23930484,  1.20086474, -0.73683366]),
                    rp=(3389.5+250)*1e3, psi=np.pi)


north_pole_bi_vec = probe.ICRF_to_BI(arrival.north_pole)

theta_star_arr_probe = np.linspace(-2, probe.theta_star_entry, 101)
pos_vec_bi_arr_probe = probe.pos_vec_bi(theta_star_arr_probe)/3389.5e3


theta_star_arr_space = np.linspace(-2, 0.0, 101)
pos_vec_bi_arr_space = space.pos_vec_bi(theta_star_arr_space)/3389.5e3

x_arr_probe = pos_vec_bi_arr_probe[0][:]
y_arr_probe = pos_vec_bi_arr_probe[1][:]
z_arr_probe = pos_vec_bi_arr_probe[2][:]

x_arr_space = pos_vec_bi_arr_space[0][:]
y_arr_space = pos_vec_bi_arr_space[1][:]
z_arr_space = pos_vec_bi_arr_space[2][:]


planet = Planet('MARS')
planet.loadAtmosphereModel('../../../atmdata/Mars/mars-gram-avg.dat', 0 , 1 ,2, 3)
planet.h_skip = 120.0E3

# Set up a vehicle object
vehicle1=Vehicle('MarsSmallSat1', 37, 20, 0.0, 1.767, 0.0, 0.1, planet)
vehicle1.setSolverParams(1E-6)
vehicle1.setDragModulationVehicleParams(20, 7.5)

# propagate a guided trajectory
# Set planet.h_low to 10 km, if vehicle dips below this level
# trajctory is terminated.
planet.h_low=10.0E3

# Set target orbit = 2000 km x 2000 km, tolerance = 20 km
# target apo intentionally set to slightly higher value to account for bias in guidance
vehicle1.setTargetOrbitParams(200.0, 2500.0, 20.0)

# Set entry phase parameters
# v_switch_kms = 2.0, lowAlt_km = 20.0,
# numPoints_lowAlt = 101, hdot_threshold = -200.0 m/s.
# These are somewhat arbitary based on experience.
vehicle1.setDragEntryPhaseParams(2.0, 20.0, 101, -200.0)

# Set beta_1 and beta_ratio
vehicle1.setDragModulationVehicleParams(20, 7.5)

# Set vehicle initial state
vehicle1.setInitialState(120.0, -89.76, -0.71, 5.3581, 1.60, -9.2475, 0.0, 0.0)
# Propogate a single vehicle trajectory
vehicle1.propogateGuidedEntryD2(1.0,1.0,0.1,2400.0)

orbiter = Orbiter(vehicle1, 200.0)
orbiter2 = PropulsiveOrbiter(approach=space, apoapsis_alt_km=70e3)


u = np.linspace(0, 2 * np.pi, 100)
v = np.linspace(0, np.pi, 100)
x = 1 * np.outer(np.cos(u), np.sin(v))
y = 1 * np.outer(np.sin(u), np.sin(v))
z = 1 * np.outer(np.ones(np.size(u)), np.cos(v))

x1 = 1.040381198513972*np.outer(np.cos(u), np.sin(v))
y1 = 1.040381198513972*np.outer(np.sin(u), np.sin(v))
z1 = 1.040381198513972*np.outer(np.ones(np.size(u)), np.cos(v))


x_ring_1 = 1.1*np.cos(u)
y_ring_1 = 1.1*np.sin(u)
z_ring_1 = 0.0*np.cos(u)

x_ring_2 = 1.2*np.cos(u)
y_ring_2 = 1.2*np.sin(u)
z_ring_2 = 0.0*np.cos(u)

mlab.figure(bgcolor=(0,0,0))
s1 = mlab.mesh(x, y, z, color=(0.7,0.3,0.0))
s2 = mlab.mesh(x1, y1, z1, color=(0.7, 0.3,0.0), opacity=0.3)
#r1 = mlab.plot3d(x_ring_1, y_ring_1, z_ring_1, color=(1,1,1), line_width=1, tube_radius=None)

p1 = mlab.plot3d(x_arr_probe, y_arr_probe, z_arr_probe, color=(1, 1, 1), line_width=3, tube_radius=None)
p2 = mlab.plot3d(x_arr_space, y_arr_space, z_arr_space, color=(0, 0, 1), line_width=3, tube_radius=None)

p3 = mlab.plot3d(orbiter.x_coast_arr,
				 orbiter.y_coast_arr,
				 orbiter.z_coast_arr, color=(1, 1, 0), line_width=3, tube_radius=None)

p4 = mlab.plot3d(orbiter.x_orbit_arr,
				 orbiter.y_orbit_arr,
				 orbiter.z_orbit_arr, color=(0, 1, 0), line_width=3, tube_radius=None)

p5 = mlab.plot3d(orbiter2.x_orbit_arr,
				 orbiter2.y_orbit_arr,
				 orbiter2.z_orbit_arr, color=(1, 0, 1), line_width=3, tube_radius=None)


mlab.show()



# SOURCE FILENAME : interplanetary.py
# AUTHOR          : Athul Pradeepkumar Girija, athulpg007@gmail.com
# DATE CREATED    : 05/13/2022, 19:02 MT
# DATE MODIFIED   : 05/13/2022, 19:03 MT
# REMARKS         : Selection of a feasible interplanetary trajectory from
#					a database for mission studies.


import pandas as pd
import matplotlib.pyplot as plt

from AMAT.launcher import Launcher

class Interplanetary:
	"""
	Stores a dataframe containing interplanetary data from an external source.

	Attributes
	----------
	ID : str
		string identifier for the interplanetary dataset
	datafile : filepath
		Excel file containing interplanetary trajectory data
	sheet_name : str
		Sheet name containing trajectory data
	df : pandas.DataFrame object
		dataframe containing the interplanetary trajectory data
	Lcdate : pandas.Series
		Series containing launch date datetime objects
	C3 : pandas.Series
		Series containing launch C3
	TOF : pandas.Series
		Series containing time of flight, years
	Avinf : pandas.Series
		Series containing arrival vinf magnitude, km/s
	"""

	def __init__(self, ID, datafile, sheet_name, Lcdate_format):
		"""

		Parameters
		----------
		ID : str
			string identifier for the interplanetary dataset
		datafile : filepath
			Excel file containing interplanetary trajectory data
		sheet_name : str
			Sheet name containing trajectory data
		Lcdate_format : str
			strftime() for launch date format e.g. "%Y%m%d"
		"""
		self.ID = ID
		self.datafile = datafile
		self.sheet_name = sheet_name

		self.df = pd.read_excel(self.datafile, sheet_name=sheet_name)
		self.Lcdate = pd.to_datetime(list(map(str, self.df['Lcdate'].values)), format=Lcdate_format)
		self.C3 = self.df["LC3"]
		self.TOF = self.df["TOF"]
		self.Avinf = self.df["Avinf"]

	def compute_launch_capability(self, launcherObj):
		"""
		Parameters
		----------
		launcherObj : AMAT.launcher.Launcher object

		Returns
		-------
		ans : numpy.ndarray
			array containing the launch mass capability for C3 values in the
			trajectory dataset
		"""
		return launcherObj.launchMass(self.C3)

	def plot_launch_mass_vs_launch_date(self, launcherObj):
		"""
		Scatter plot of the launch capability as a function of launch date.

		Parameters
		----------
		launcherObj : AMAT.launcher.Launcher object
			AMAT Launcher object

		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter of the launch capability as a function of launch date
		"""
		Lcmass = self.compute_launch_capability(launcherObj)

		plt.figure(figsize=(8,6))
		plt.plot(self.Lcdate, Lcmass, marker='o', color='g', linestyle='None')
		plt.title(self.ID+" : "+launcherObj.ID, fontsize=14)
		plt.xlabel(r'Launch Year', fontsize=14)
		plt.ylabel("Launch capability, kg", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')
		plt.ylim(max(0, sorted(set(Lcmass))[1])-100, max(Lcmass)+100)

		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/'+self.ID+"_"+launcherObj.ID+"_"+"launch_mass_vs_launch_date"+'.png', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/'+self.ID+"_"+launcherObj.ID+"_"+"launch_mass_vs_launch_date"+'.pdf', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/'+self.ID+"_"+launcherObj.ID+"_"+"launch_mass_vs_launch_date"+'.eps', dpi=300, bbox_inches='tight')

		plt.show()

	def plot_TOF_vs_launch_date(self):
		"""
		Scatter plot of the time of flight as a function of launch date.

		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter of the launch capability as a function of launch date
		"""

		plt.figure(figsize=(8,6))
		plt.plot(self.Lcdate, self.TOF, marker='o', color='g', linestyle='None')
		plt.title(self.ID, fontsize=14)
		plt.xlabel(r'Launch Year', fontsize=14)
		plt.ylabel("TOF, years", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')

		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/' + self.ID + "_" + "TOF_vs_launch_date" + '.png', dpi=300,  bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "TOF_vs_launch_date" + '.pdf', dpi=300,  bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "TOF_vs_launch_date" + '.eps', dpi=300,  bbox_inches='tight')

		plt.show()

	def plot_Avinf_vs_launch_date(self):
		"""
		Scatter plot of the arrival vinf magnitude as a function of launch date.

		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter of the arrival vinf magnitude as a function of launch date
		"""

		plt.figure(figsize=(8,6))
		plt.plot(self.Lcdate, self.Avinf, marker='o', color='g', linestyle='None')
		plt.title(self.ID, fontsize=14)
		plt.xlabel(r'Launch Year', fontsize=14)
		plt.ylabel(r"Arrival $V_{\infty}$, km/s", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')

		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date" + '.png', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date" + '.pdf', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date" + '.eps', dpi=300, bbox_inches='tight')

		plt.show()

	def plot_Avinf_vs_launch_date_with_launch_mass_colorbar(self, launcherObj, scale=20):
		"""
		Scatter plot of the arrival vinf magnitude as a function of launch date
		with launch capability colorbar.

		Parameters
		-------
		launcherObj : AMAT.launcher.Launcher object
			AMAT Launcher object
		scale : float
			scatter plot marker size, defaults to 20


		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter of the arrival vinf magnitude as a function of launch date
			with launch capability colorbar.

		"""

		Lcmass = self.compute_launch_capability(launcherObj)

		plt.figure(figsize=(8,6))
		plt.scatter(self.Lcdate, self.Avinf, c=Lcmass, marker='o', cmap='jet',
					vmin = max(0, sorted(set(Lcmass))[1]), vmax = max(Lcmass), s=scale)
		cbar = plt.colorbar()
		cbar.ax.tick_params(labelsize=14)
		cbar.set_label(r'Launch capability, kg', labelpad=-20, y=1.05, rotation=0, fontsize=14)
		cbar.ax.tick_params(axis='y', direction='in')
		plt.title(self.ID, fontsize=14)
		plt.xlabel(r'Launch Year', fontsize=14)
		plt.ylabel(r"Arrival $V_{\infty}$, km/s", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')

		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date_launch_mass_colorbar" + '.png', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date_launch_mass_colorbar" + '.pdf', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "Avinf_vs_launch_date_launch_mass_colorbar" + '.eps', dpi=300, bbox_inches='tight')

		plt.show()



	def plot_launch_mass_vs_TOF(self, launcherObj):
		"""
		Scatter plot of the launch mass as a function of TOF.

		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter plot of the launch mass as a function of TOF.
		"""
		Lcmass = self.compute_launch_capability(launcherObj)

		plt.figure(figsize=(8,6))
		plt.plot(self.TOF, Lcmass, marker='o', color='g', linestyle='None')
		plt.title(self.ID+" : "+launcherObj.ID, fontsize=14)
		plt.xlabel(r'TOF, years', fontsize=14)
		plt.ylabel("Launch capability, kg", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')
		plt.ylim(max(0, sorted(set(Lcmass))[1])-100, max(Lcmass)+100)


		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/' + self.ID + "_" + launcherObj.ID + "_" + "launch_mass_vs_TOF" + '.png', dpi=300, box_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + launcherObj.ID + "_" + "launch_mass_vs_TOF" + '.pdf', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + launcherObj.ID + "_" + "launch_mass_vs_TOF" + '.eps', dpi=300, bbox_inches='tight')

		plt.show()



	def plot_launch_mass_vs_TOF_with_vinf_colorbar(self, launcherObj, scale=20):
		"""
		Scatter plot of the launch mass as a function of TOF
		with arrival vinf colorbar.

		Parameters
		-------
		launcherObj : AMAT.launcher.Launcher object
			AMAT Launcher object
		scale : float
			scatter plot marker size, defaults to 20


		Returns
		-------
		ans : matplotlib.figure.Figure
			Scatter plot of the launch mass as a function of TOF
			with arrival vinf colorbar.

		"""

		Lcmass = self.compute_launch_capability(launcherObj)

		plt.figure(figsize=(8,6))
		plt.scatter(self.TOF, Lcmass, c=self.Avinf , marker='o', cmap='jet',
					vmin=self.Avinf.min(), vmax=self.Avinf.max(), s=scale)
		cbar = plt.colorbar()
		cbar.ax.tick_params(labelsize=14)
		cbar.set_label(r'$V_{\infty}$, km/s', labelpad=-20, y=1.05, rotation=0, fontsize=14)
		cbar.ax.tick_params(axis='y', direction='in')
		plt.title(self.ID+" : "+launcherObj.ID, fontsize=14)
		plt.xlabel(r'TOF, years', fontsize=14)
		plt.ylabel("Launch capability, kg", fontsize=14)
		plt.xticks(fontsize=14)
		plt.yticks(fontsize=14)
		ax = plt.gca()
		ax.tick_params(direction='in')
		ax.yaxis.set_ticks_position('both')
		ax.xaxis.set_ticks_position('both')
		plt.ylim(max(0, sorted(set(Lcmass))[1])-100, max(Lcmass)+100)


		plt.legend(loc='upper right', fontsize=12, framealpha=0.8)

		ax.xaxis.set_tick_params(direction='in', which='both')
		ax.yaxis.set_tick_params(direction='in', which='both')

		ax.xaxis.set_tick_params(width=2, length=8)
		ax.yaxis.set_tick_params(width=2, length=8)

		ax.xaxis.set_tick_params(width=1, length=6, which='minor')
		ax.yaxis.set_tick_params(width=1, length=6, which='minor')

		ax.xaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.xaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		ax.yaxis.grid(which='major', color='k', linestyle='dotted', linewidth=0.5)
		ax.yaxis.grid(which='minor', color='k', linestyle='dotted', linewidth=0.0)

		for axis in ['top', 'bottom', 'left', 'right']:
			ax.spines[axis].set_linewidth(2)

		plt.savefig('../plots/' + self.ID + "_" + "launch_mass_vs_TOF_Avinf_colorbar" + '.png', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "launch_mass_vs_TOF_Avinf_colorbar" + '.pdf', dpi=300, bbox_inches='tight')
		plt.savefig('../plots/' + self.ID + "_" + "launch_mass_vs_TOF_Avinf_colorbar" + '.eps', dpi=300, bbox_inches='tight')

		plt.show()




from mayavi import mlab
import numpy as np

from AMAT.approach import Approach

probe1 = Approach("URANUS",
						v_inf_vec_icrf_kms=np.array([-9.62521831, 16.51192666,  7.46493598]),
						rp=(25559+250)*1e3, psi=np.pi,
						is_entrySystem=True, h_EI=1000e3)

theta_star_arr_probe1 = np.linspace(-1.8, probe1.theta_star_entry, 101)
pos_vec_bi_arr_probe1 = probe1.pos_vec_bi(theta_star_arr_probe1)/25559e3


x_arr_probe1 = pos_vec_bi_arr_probe1[0][:]
y_arr_probe1 = pos_vec_bi_arr_probe1[1][:]
z_arr_probe1 = pos_vec_bi_arr_probe1[2][:]



u = np.linspace(0, 2 * np.pi, 100)
v = np.linspace(0, np.pi, 100)
x = 1*np.outer(np.cos(u), np.sin(v))
y = 1*np.outer(np.sin(u), np.sin(v))
z = 1*np.outer(np.ones(np.size(u)), np.cos(v))

x1 = 1.040381198513972*np.outer(np.cos(u), np.sin(v))
y1 = 1.040381198513972*np.outer(np.sin(u), np.sin(v))
z1 = 1.040381198513972*np.outer(np.ones(np.size(u)), np.cos(v))


x_ring_1 = 1.1*np.cos(u)
y_ring_1 = 1.1*np.sin(u)
z_ring_1 = 0.0*np.cos(u)

x_ring_2 = 1.2*np.cos(u)
y_ring_2 = 1.2*np.sin(u)
z_ring_2 = 0.0*np.cos(u)

mlab.figure(bgcolor=(0,0,0))
s1 = mlab.mesh(x, y, z, color=(0.10,0.55,0.35))
s2 = mlab.mesh(x1, y1, z1, color=(0.2, 0.4, 0.4), opacity=0.3)
r1 = mlab.plot3d(x_ring_1, y_ring_1, z_ring_1, color=(1,1,1), line_width=1, tube_radius=None)
r2 = mlab.plot3d(x_ring_2, y_ring_2, z_ring_2, color=(1,1,1), line_width=1, tube_radius=None)

p1 = mlab.plot3d(x_arr_probe1, y_arr_probe1, z_arr_probe1, color=(1,0,0), line_width=3, tube_radius=None)

mlab.show()


# SOURCE FILENAME : approach.py
# AUTHOR          : Athul Pradeepkumar Girija, apradee@purdue.edu
# DATE CREATED    : 08/12/2022, 08:03 MT
# DATE MODIFIED   : 05/01/2022 16:14 MT
# REMARKS         : Compute the probe/spacecraft arrival trajectory
#                   following Kyle's dissertation Chapter 2.
# REFERENCE 	  : Hughes, Ph.D. Dissertation, 2016.

import numpy as np
from numpy import linalg as LA
from AMAT.planet import Planet

class Approach:
	"""
	Compute the probe/spacecraft approach trajectory for a given
	v_inf_vec, periapsis radius, psi.

	REF: Hughes (2016), Ph.D. Dissertation, Purdue University

	Attributes
	----------
	planetObj : AMAT.planet.Planet object
		arrival planet object for approach traje
	a : float
		semi-major axis, meters
	e : float
		eccentricity
	beta : float
		true anomaly of the outgoing asymptote, rad
	v_inf_vec_bi_kms : numpy.ndarray
		v_inf vector in body-inertial frame, km/s
	v_inf_vec_bi : numpy.ndarray
		v_inf vector in body-inertial frame, m/s
	v_inf_vec_bi_mag_kms : float
		v_inf magnitude in body-inertial frame, km/s
	phi_1 : float
		rad, as defined in Eq. (2.16) from REF.
	phi_2 : float
		rad, as defined in Eq. (2.17) from REF.
	rp_vec_bi_unit : numpy.ndarray
		periapsis radius unit vector in body-inertial frame
	rp_vec_bi : numpy.ndarray
		periapsis radius vector in body-inertial frame, meters
	e_vec_bi_unit : numpy.ndarray
		eccentricity unit vector in body-inertial frame
	e_vec_bi : numpy.ndarray
		eccentricity vector in body-inertial frame, meters
	h : float
		angular momentum, SI units
	h_vec_bi : numpy.ndarray
		angular momentum vector in body-inertial frame, SI units
	h_vec_bi_unit : numpy.ndarray
		angular momentum unit vector in body-inertial frame, SI units
	i : float
		inclination [0, np.pi], rad
	N_vec_bi : numpy.ndarray
		node vector in body-inertial frame
	N_vec_bi_unit : numpy.ndarray
		node unit vector in body-inertial frame
	OMEGA : float
		right asecension of ascending node, rad
	omega : float
		argument of periapsis, rad
	N_ref_bi : numpy.ndarray
		reference normal vector in body-inertial frame (planet's spin axis)
	S_vec_bi : numpy.ndarray
		S vector in body-inertial frame, meters
	S_vec_bi_unit : numpy.ndarray
		S unit vector in body-inertial frame, meters
	T_vec_bi : numpy.ndarray
		T vector in body-inertial frame, meters
	T_vec_bi_unit : numpy.ndarray
		T unit vector in body-inertial frame, meters
	R_vec_bi : numpy.ndarray
		R vector in body-inertial frame, meters
	R_vec_bi_unit : numpy.ndarray
		R unit vector in body-inertial frame, meters
	B_vec_bi : numpy.ndarray
		B vector in body-inertial frame, meters
	B_vec_bi_unit : numpy.ndarray
		B unit vector in body-inertial frame, meters
	b_mag : float
		aim point vector B magnitude, meters
	b_plane_angle_theta : float
		angle between aim point vector B and T
	r_EI : float
		atmospheric entry interface radius, m
	theta_star_entry : float
		true anomaly at entry interface, rad
	r_vec_entry_bi : numpy.ndarray
		position vector at entry interface in body-inertial frame, meters
	r_vec_entry_bi_unit : numpy.ndarray
		position unit vector at entry interface in body-inertial frame, meters
	r_vec_entry_bi_mag : float
		position vector magnitude at entry interface in body-inertial frame, meters
	v_entry_inertial_mag : float
		inertial velocity magnitude at entry interface in body-inertial frame, m/s
	gamma_entry_inertial : float
		inertial flight path angle at entry interface, rad
	v_vec_entry_bi : numpy.ndarray
		inertial velocity vector at entry interface in body-inertial frame, m/s
	v_vec_entry_bi_unit : numpy.ndarray
		inertial velocity unit vector at entry interface in body-inertial frame, m/s
	gamma_entry_inertial_check : float
		analytic solution for inertial flight path angle at entry interface, rad
	latitude_entry_bi : float
		entry latitude in body-inertial frame, rad
	longitude_entry_bi : float
		entry longitude in body-inertial frame, rad
	v_vec_entry_atm : numpy.ndarray
		atmosphere-relative velocity vector at entry interface in body-inertial frame, m/s
	v_entry_atm_mag : float
		atmosphere-relative velocity magnitude at entry interface in body-inertial frame, m/s
	v_vec_entry_atm_unit : numpy.ndarray
		atmosphere-relative velocity unit vector at entry interface in body-inertial frame, m/s
	gamma_entry_atm: float
		atmosphere-relative flight path angle at entry interface, rad
	v_vec_entry_atm_unit_proj : numpy.ndarray
		projection of atmosphere-relative vector onto local horizontal plane at entry interface
		in body-inertial frame
	local_latitude_parallel_vec_unit : numpy.ndarray
		unit vector direction of local parallel of latitude at entry interface in
		body-inertia frame
	heading_entry_atm : float
		atmosphere-relative heading angle at entry interface, rad
	"""

	def __init__(self, arrivalPlanet, v_inf_vec_icrf_kms, rp, psi,
				       is_entrySystem=False, h_EI=None):
		"""

		Initializes the Approach class with the following params

		Parameters
		----------
		arrivalPlanet : strarrivalPlanet : str
			Name of the planetary body, must be all uppercase
			Valid entries are: 'VENUS', 'EARTH', 'MARS',
			'JUPITER', 'SATURN', 'TITAN', 'URANUS', 'NEPTUNE'
		v_inf_vec_ICRF_kms : numpy.ndarray
			v_inf vector in ICRF, km/s
		rp : float
			target periapsis radius, meters
		psi : float
			angular position on ring of periapsis radius, rad
			following Hughes' Ph.D. dissertation (Purdue, 2016)
		is_entrySystem : bool, optional
			must be set to True if approach trajectory is for
			an atmospheric entry system (eg: probe or aerocapture vehicle).
			must be set to False for propulsive insertion orbiter which
			remains outside the atmosphere. False by default
		h_EI : float
			atmospheric entry interface altitude, meters
			must be provided if is_entrySystem is set to True
		"""

		self.planetObj = Planet(arrivalPlanet)
		self.a0 = self.planetObj.a0
		self.d0 = self.planetObj.d0

		self.v_inf_vec_icrf_kms = v_inf_vec_icrf_kms
		self.rp = rp
		self.psi = psi

		self.v_inf_mag_kms = LA.norm(self.v_inf_vec_icrf_kms)
		self.v_inf_mag = self.v_inf_mag_kms*1e3

		# Equations (2.12), (2.13), (2.14)
		self.a = -self.planetObj.GM/(self.v_inf_mag**2)
		self.e = 1 - self.rp/self.a
		self.beta = np.arccos(1.0/self.e)

		# Equation (2.3)
		self.v_inf_vec_bi_kms = self.ICRF_to_BI(self.v_inf_vec_icrf_kms)
		self.v_inf_vec_bi = self.v_inf_vec_bi_kms * 1e3
		self.v_inf_vec_bi_mag_kms = LA.norm(self.v_inf_vec_bi_kms)
		self.v_inf_vec_bi_unit = self.v_inf_vec_bi_kms / LA.norm(self.v_inf_vec_bi_kms)

		# Equation (2.16)
		self.phi_1 = np.arctan2(self.v_inf_vec_bi[1], self.v_inf_vec_bi[0])
		self.v_inf_vec_bi_prime = (np.matmul(self.R3(self.phi_1), self.v_inf_vec_bi.T)).T

		# Equation (2.17)
		self.phi_2 = np.arctan2(self.v_inf_vec_bi_prime[0], self.v_inf_vec_bi_prime[2])
		self.phi_2_analytic = np.arctan2((self.v_inf_vec_bi[0] * np.cos(self.phi_1) + \
									     self.v_inf_vec_bi[1] * np.sin(self.phi_1)), \
								         self.v_inf_vec_bi[2])

		self.rp_vec_bi_x_unit = np.cos(self.phi_1)*(np.sin(self.beta) * np.cos(self.psi) * np.cos(self.phi_2) + np.cos(self.beta) * np.sin(self.phi_2)) - \
								np.sin(self.phi_1) * np.sin(self.beta) * np.sin(self.psi)
		self.rp_vec_bi_y_unit = np.sin(self.phi_1)*(np.sin(self.beta) * np.cos(self.psi) * np.cos(self.phi_2) + np.cos(self.beta) * np.sin(self.phi_2)) + \
								np.cos(self.phi_1) * np.sin(self.beta) * np.sin(self.psi)
		self.rp_vec_bi_z_unit = np.cos(self.beta) * np.cos(self.phi_2) - np.sin(self.beta) * np.cos(self.psi) * np.sin(self.phi_2)

		self.rp_vec_bi_unit = np.array([self.rp_vec_bi_x_unit,
										self.rp_vec_bi_y_unit,
										self.rp_vec_bi_z_unit])

		self.rp_vec_bi = self.rp * self.rp_vec_bi_unit

		self.rp_vec_bi_dprime = self.BI_to_BI_dprime(self.rp_vec_bi)
		self.rp_vec_bi_dprime_analytic = self.rp * np.array([np.sin(self.beta) * np.cos(self.psi),
													np.sin(self.beta) * np.sin(self.psi),
													np.cos(self.beta)])

		self.e_vec_bi = self.e * self.rp_vec_bi_unit
		self.e_vec_bi_unit = self.e_vec_bi / LA.norm(self.e_vec_bi)

		self.h = np.sqrt(self.a * self.planetObj.GM * (1 - self.e**2))

		self.h_vec_bi = np.cross(self.rp_vec_bi, self.v_inf_vec_bi)
		self.h_vec_bi_unit = self.h_vec_bi / LA.norm(self.h_vec_bi)

		self.i = np.arccos(self.h_vec_bi_unit[2])

		self.N_vec_bi = np.cross(np.array([0, 0, 1]), self.h_vec_bi)
		self.N_vec_bi_unit = self.N_vec_bi / LA.norm(self.N_vec_bi)

		if self.N_vec_bi_unit[1] >= 0:
			self.OMEGA = np.arccos(self.N_vec_bi_unit[0])
		else:
			self.OMEGA = 2*np.pi - np.arccos(self.N_vec_bi_unit[0])

		if self.e_vec_bi_unit[2] >= 0:
			self.omega = np.arccos(np.dot(self.N_vec_bi_unit, self.e_vec_bi_unit))
		else:
			self.omega = 2*np.pi - np.arccos(np.dot(self.N_vec_bi_unit, self.e_vec_bi_unit))

		self.N_ref_bi = np.array([0, 0, 1])
		self.S_vec_bi = self.v_inf_vec_bi
		self.S_vec_bi_unit = self.v_inf_vec_bi_unit

		self.T_vec_bi = np.cross(self.S_vec_bi, self.N_ref_bi)
		self.T_vec_bi_unit = self.T_vec_bi / LA.norm(self.T_vec_bi)
		self.T_vec_bi_unit_dprime = self.BI_to_BI_dprime(self.T_vec_bi_unit)

		self.R_vec_bi = np.cross(self.S_vec_bi, self.T_vec_bi)
		self.R_vec_bi_unit = self.R_vec_bi / LA.norm(self.R_vec_bi)
		self.R_vec_bi_unit_dprime = self.BI_to_BI_dprime(self.R_vec_bi_unit)

		self.b_mag = abs(self.a) * np.sqrt(self.e ** 2 - 1)
		self.b_plane_angle_theta = self.psi + np.pi / 2

		self.B_vec_bi = np.cross(self.S_vec_bi, self.h_vec_bi)
		self.B_vec_bi_unit = self.B_vec_bi / LA.norm(self.B_vec_bi)


		if is_entrySystem == True:
			self.h_EI = h_EI
			self.r_EI = self.planetObj.RP + self.h_EI

			self.theta_star_entry = -1 * np.arccos(((self.h ** 2 / (self.planetObj.GM * self.r_EI)) - 1) * (1.0 / self.e))

			self.r_vec_entry_bi = self.pos_vec_bi(self.theta_star_entry)
			self.r_vec_entry_bi_unit = self.r_vec_entry_bi / LA.norm(self.r_vec_entry_bi)
			self.r_vec_entry_bi_mag = LA.norm(self.r_vec_entry_bi)

			self.v_entry_inertial_mag = np.sqrt(self.v_inf_mag**2 + 2 * self.planetObj.GM / self.r_EI)

			self.gamma_entry_inertial = -1 * np.arccos(self.h / (self.r_EI * self.v_entry_inertial_mag))

			self.v_vec_entry_bi = self.vel_vec_bi(self.theta_star_entry)
			self.v_vec_entry_bi_unit = self.v_vec_entry_bi / LA.norm(self.v_vec_entry_bi)

			self.gamma_entry_inertial_check = np.pi / 2 - \
											  np.arccos(np.dot(self.r_vec_entry_bi_unit, self.v_vec_entry_bi_unit))

			self.latitude_entry_bi = np.arcsin(self.r_vec_entry_bi_unit[2])
			self.longitude_entry_bi = np.arctan(self.r_vec_entry_bi_unit[1] / self.r_vec_entry_bi_unit[0])

			self.xi_1 = np.arctan(self.r_vec_entry_bi_unit[1]/self.r_vec_entry_bi_unit[0])
			self.latitude_entry_bi_analytic = np.arctan(self.r_vec_entry_bi_unit[2]/
														(self.r_vec_entry_bi_unit[0]*np.cos(self.xi_1) + self.r_vec_entry_bi_unit[1]*np.sin(self.xi_1)))

			self.v_vec_entry_atm = self.vel_vec_entry_atm()
			self.v_entry_atm_mag = LA.norm(self.v_vec_entry_atm)
			self.v_vec_entry_atm_unit = self.v_vec_entry_atm / self.v_entry_atm_mag

			self.gamma_entry_atm = np.pi / 2 - \
								   np.arccos(np.dot(self.r_vec_entry_bi_unit, self.v_vec_entry_atm_unit))

			self.v_vec_entry_atm_unit_proj = self.v_vec_entry_atm_unit - \
										     np.dot(self.v_vec_entry_atm_unit, self.r_vec_entry_bi_unit)*self.r_vec_entry_bi_unit


			self.local_latitude_parallel_vec_unit = np.array([ -np.sin(self.longitude_entry_bi),
															    np.cos(self.longitude_entry_bi),
															    0.0])

			self.heading_entry_atm = np.arccos(np.dot(self.v_vec_entry_atm_unit_proj, self.local_latitude_parallel_vec_unit))

		else:
			self.theta_star_periapsis = 0
			self.r_vec_rp_bi = self.rp_vec_bi
			self.v_vec_rp_bi = self.vel_vec_bi(self.theta_star_periapsis)


	def R1(self, theta):
		"""
		Direction cosine matrix for rotation about x-axis.

		Parameters
		----------
		theta : float
			rotation angle about x-axis

		Returns
		-------
		ans : numpy.ndarray
			Direction cosine matrix for rotation about x-axis.

		"""
		return np.array([[1, 0, 0],
						 [0, np.cos(theta), np.sin(theta)],
						 [0, -np.sin(theta), np.cos(theta)]])

	def R2(self, theta):
		"""
		Direction cosine matrix for rotation about y-axis.

		Parameters
		----------
		theta : float
			rotation angle about y-axis

		Returns
		-------
		ans : numpy.ndarray
			Direction cosine matrix for rotation about y-axis.

		"""
		return np.array([[np.cos(theta), 0, -np.sin(theta)],
						 [0, 1, 0],
						 [np.sin(theta), 0, np.cos(theta)]])

	def R3(self, theta):
		"""
		Direction cosine matrix for rotation about z-axis.

		Parameters
		----------
		theta : float
			rotation angle about zaxis

		Returns
		-------
		ans : numpy.ndarray
			Direction cosine matrix for rotation about z-axis.

		"""
		return np.array([[np.cos(theta), np.sin(theta), 0],
						 [-np.sin(theta), np.cos(theta), 0],
						 [0, 0, 1]])

	def ICRF_to_BI(self, X_ICRF):
		"""
		Converts an input vector from ICRF to body-inertial frame

		Parameters
		----------
		X_ICRF : numpy.ndarray
			row vector in ICRF frame

		Returns
		-------
		ans : numpy.ndarray
			input vector in body-inertial frame

		"""
		R1R3 = np.matmul(self.R1(np.pi/2 - self.d0), self.R3(np.pi/2 + self.a0))
		return np.matmul(R1R3, X_ICRF.T).T

	def BI_to_BI_dprime(self, X_ICRF):
		"""
		Converts an input vector from body-inertial to body-inertial-double-prime
		frame following Eq. (2.10) of REF.

		Parameters
		----------
		X_ICRF : numpy.ndarray
			row vector in body-inertial frame

		Returns
		-------
		ans : numpy.ndarray
			input vector in body-inertial double-prime frame

		"""
		R2R3 = np.matmul(self.R2(self.phi_2), self.R3(self.phi_1))
		return np.matmul(R2R3, X_ICRF.T).T

	def pos_vec_bi(self, theta_star):
		"""
		Computes the position vector in body-inertial frame
		for a specified true-anomaly.

		Parameters
		----------
		theta_star : float
			true anomaly, rad

		Returns
		-------
		pos_vec_bi : numpy.ndarray
			position vector in body-inertial frame, meters
		"""

		r = (self.h**2 / self.planetObj.GM) / (1 + self.e*np.cos(theta_star))
		theta = theta_star + self.omega

		rx_unit = np.cos(self.OMEGA)*np.cos(theta) - np.sin(self.OMEGA)*np.cos(self.i)*np.sin(theta)
		ry_unit = np.sin(self.OMEGA)*np.cos(theta) + np.cos(self.OMEGA)*np.cos(self.i)*np.sin(theta)
		rz_unit = np.sin(self.i)*np.sin(theta)

		pos_vec_bi = r*np.array([rx_unit, ry_unit, rz_unit])
		return pos_vec_bi


	def pos_vec_bi_dprime(self, theta_star):

		"""
		Computes the position vector in body-inertial double-prime
		frame for a specified true-anomaly.

		Parameters
		----------
		theta_star : float
			true anomaly, rad

		Returns
		-------
		pos_vec_bi_dprime : numpy.ndarray
			position vector in body-inertial double-prime frame, meters
		"""

		pos_vec_bi = self.pos_vec_bi(theta_star)

		pos_vec_bi_dprime = (np.matmul(self.R2(self.phi_2),
		        			 np.matmul(self.R3(self.phi_1), pos_vec_bi))).T

		return pos_vec_bi_dprime


	def r_mag_bi(self, theta_star):
		"""
		Computes the position vector magnitude in body-inertial frame at a given true anomaly.

		Parameters
		----------
		theta_star : float
			true anomaly, rad

		Returns
		-------
		r_mag_bi : float
			position vector magnitude, meters

		"""
		r_mag_bi = (self.h**2 / self.planetObj.GM) / (1 + self.e*np.cos(theta_star))
		return r_mag_bi

	def vel_vec_bi(self, theta_star):
		"""
		Computes the velocity vector magnitude in body-inertial frame at a given true anomaly.

		Parameters
		----------
		theta_star : float
			true anomaly, rad

		Returns
		-------
		vel_vec_bi : numpy.ndarray
			velocity vector in body-inertial frame, m/s

		"""

		r_mag_bi = self.r_mag_bi(theta_star)
		v_mag_bi = np.sqrt((self.v_inf_mag)**2 + 2*self.planetObj.GM/r_mag_bi)
		gamma = -1*np.arccos(self.h/(r_mag_bi*v_mag_bi))

		vr = v_mag_bi*np.sin(gamma)
		vt = v_mag_bi*np.cos(gamma)
		theta = theta_star + self.omega

		vx = vr*( np.cos(theta)*np.cos(self.OMEGA) - np.sin(theta)*np.cos(self.i)*np.sin(self.OMEGA)) +\
		     vt*(-np.sin(theta)*np.cos(self.OMEGA) - np.cos(theta)*np.cos(self.i)*np.sin(self.OMEGA))

		vy = vr*( np.cos(theta)*np.sin(self.OMEGA) + np.sin(theta)*np.cos(self.i)*np.cos(self.OMEGA)) +\
		     vt*( np.cos(theta)*np.cos(self.i)*np.cos(self.OMEGA) - np.sin(theta)*np.sin(self.OMEGA))

		vz = vr*np.sin(theta)*np.sin(self.i) + vt*np.cos(theta)*np.sin(self.i)

		vel_vec_bi = np.array([vx, vy, vz])
		return vel_vec_bi

	def vel_vec_entry_atm(self):
		"""
		Computes the atmosphere-relative velocity vector magnitude in body-inertial frame
		at atmospheric entry interface.

		Parameters
		----------

		Returns
		-------
		vel_vec_entry_atm : numpy.ndarray
			atmosphere-relative velocity vector in body-inertial frame, at
			atmospheric entry interface, m/s

		"""
		v_entry_atm_x = self.v_vec_entry_bi[0] - \
		          self.r_vec_entry_bi_mag * self.planetObj.OMEGA * \
		          np.cos(self.latitude_entry_bi) * (-np.sin(self.longitude_entry_bi))
		v_entry_atm_y = self.v_vec_entry_bi[1] - \
		          self.r_vec_entry_bi_mag * self.planetObj.OMEGA * \
		          np.cos(self.latitude_entry_bi) * (np.cos(self.longitude_entry_bi))
		v_entry_atm_z = self.v_vec_entry_bi[2]

		vel_vec_entry_atm = np.array([v_entry_atm_x, v_entry_atm_y, v_entry_atm_z])
		return vel_vec_entry_atm



# SOURCE FILENAME : manuever.py
# AUTHOR          : Athul Pradeepkumar Girija, apradee@purdue.edu
# DATE MODIFIED   : 04/29/2022, 08:27 MT
# REMARKS         : Compute the deflection maneuver delta-v
#                   following Kyle's dissertation Chapter 2.
# REFERENCE 	  : Hughes, Ph.D. Dissertation, 2016.

import numpy as np
from numpy import linalg as LA

from AMAT.planet import Planet
from AMAT.approach import Approach

class ProbeOrbiterDeflection:
	"""
	Computes the deflection manuever delta-V and time of flight for an orbiter spacecraft
	divert manuever for propulsive orbit insertion after probe release on approach.

	Attributes
	----------
	planetObj : AMAT.planet.Planet object
		arrival planet object for approach trajectory
	probe : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the entry probe
	space : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the orbiter spacecraft
	r_dv_rp : float
		radial distance at which the deflection maneuver is
		performed, in terms of arrival planet radius
		Example: 1000, for maneuver performed at radial distance of
		1000 times the radius of the planet.
	r_dv : float
		radial distance at which the deflection maneuver is
		performed, in meters
	theta_star_dv_probe : float
		true anomaly of the probe at deflection maneuver, rad
	r_vec_dv : numpy.ndarray
		probe position vector at maneuver in body-inertial frame, meters
	r_vec_dv_unit : numpy.ndarray
		probe position unit vector at maneuver in body-inertial frame
	delta_theta_star_probe : float
		true anomaly change for probe (Eq. 2.75, REF), rad
	delta_theta_star_space : float
		true anomaly change for orbiter (Eq. 2.78, REF), rad
	v_vec_dv_probe : numpy.ndarray
		probe velocity vector in body-inertial frame, meters
	v_vec_dv_space : numpy.ndarray
		orbiter velocity vector in body-inertial frame, meters
	v_vec_dv_maneuver : numpy.ndarray
		orbiter deflection maneuver delta-V vector, m/s
	v_vec_dv_maneuver_mag : numpy.ndarray
		orbiter deflection maneuver delta-V magnitude, m/s
	TOF_probe : float
		probe time of flight from deflection maneuver until
		atmospheric entry interface, days
	TOF_space: float
		orbiter time of flight from deflection maneuver until
		periapsis, days

	"""

	def __init__(self, arrivalPlanet, v_inf_vec_icrf_kms,
								  rp_probe, psi_probe,  h_EI_probe,
								  rp_space, psi_space,
								  r_dv_rp):
		"""

		Initializes the ProbeOrbiterDeflection class with the following params

		Parameters
		----------
		arrivalPlanet :  str
			Name of the planetary body, must be all uppercase
			Valid entries are: 'VENUS', 'EARTH', 'MARS',
			'JUPITER', 'SATURN', 'TITAN', 'URANUS', 'NEPTUNE'
		v_inf_vec_ICRF_kms : numpy.ndarray
			v_inf vector in ICRF, km/s
		rp_probe : float
			target periapsis radius for entry probe, meters
		psi_probe : float
			target angular position on ring of periapsis radius
			for entry probe, rad
		h_EI_probe : float
			atmospheric entry interface altitude for entry
			probe, meters
		rp_space : float
			target periapsis radius for spacecraft, meters
		psi_space : float
			target angular position on ring of periapsis radius
			for orbiter spacecraft, rad
		r_dv_rp : float
			radial distance at which the deflection manuever is
			performed, in terms of arrival planet radius
			Example: 1000, for manuever performed at radial distance of
			1000 times the radius of the planet.
		"""

		self.planetObj = Planet(arrivalPlanet)
		self.probe = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_probe, psi_probe, is_entrySystem=True, h_EI=h_EI_probe)
		self.space = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_space, psi_space)
		self.r_dv_rp = r_dv_rp
		self.r_dv = self.r_dv_rp * self.planetObj.RP

		self.theta_star_dv_probe = -1*np.arccos(((self.probe.h**2/(self.planetObj.GM * self.r_dv)) - 1)*\
												(1.0/self.probe.e))

		self.r_vec_dv = self.probe.pos_vec_bi(self.theta_star_dv_probe)
		self.r_vec_dv_unit = self.r_vec_dv / LA.norm(self.r_vec_dv)

		self.delta_theta_star_probe = abs(self.theta_star_dv_probe)
		self.delta_theta_star_space = np.arccos(np.dot(self.space.rp_vec_bi_unit, self.r_vec_dv_unit))

		self.P_probe = self.probe.a*(1 - self.probe.e**2)

		self.f_probe = 1 - (self.probe.rp/self.P_probe)*(1 - np.cos(self.delta_theta_star_probe))
		self.g_probe = ((self.r_dv*self.probe.rp)/(np.sqrt(self.planetObj.GM*self.P_probe)))*np.sin(self.delta_theta_star_probe)

		self.v_vec_dv_probe = (self.probe.rp_vec_bi - self.f_probe*self.r_vec_dv)/self.g_probe

		self.C = np.sqrt(self.space.rp**2 + self.r_dv**2 - 2*self.space.rp*self.r_dv*np.cos(self.delta_theta_star_space))
		self.S = 0.5*(self.space.rp + self.r_dv + self.C)
		self.alpha_prime = 2*np.arcsinh(np.sqrt(self.S/(2*abs(self.space.a))))
		self.beta_prime  = 2*np.arcsinh(np.sqrt((self.S-self.C)/(2*abs(self.space.a))))

		self.P1 = (4*abs(self.space.a)*(self.S - self.space.rp)*(self.S - self.r_dv))/(self.C**2)
		self.P2 = np.sinh(0.5*(self.alpha_prime + self.beta_prime))

		self.P_space = self.P1*self.P2**2

		self.f_space = 1 - (self.space.rp / self.P_space) * (1 - np.cos(self.delta_theta_star_space))
		self.g_space = ((self.r_dv * self.space.rp) / (np.sqrt(self.planetObj.GM * self.P_space))) * np.sin(self.delta_theta_star_space)

		self.v_vec_dv_space = (self.space.rp_vec_bi - self.f_space * self.r_vec_dv) / self.g_space

		self.dv_maneuver_vec = self.v_vec_dv_space - self.v_vec_dv_probe
		self.dv_maneuver_mag = LA.norm(self.dv_maneuver_vec)

		self.H1 = np.sqrt((self.probe.e-1)/(self.probe.e+1))*np.tan(0.5*self.delta_theta_star_probe)
		self.H_dv_probe = 2*np.arctanh(self.H1)

		self.T1 = np.sqrt(abs(self.probe.a)**3/self.planetObj.GM)
		self.T2 = abs(self.probe.e*np.sinh(self.H_dv_probe) - self.H_dv_probe)

		self.TOF_probe = (self.T1*self.T2) / 86400

		self.U1 = np.sqrt(abs(self.space.a)**3/self.planetObj.GM)
		self.U2 = (np.sinh(self.alpha_prime) - self.alpha_prime) - (np.sinh(self.beta_prime) - self.beta_prime)

		self.TOF_space = (self.U1*self.U2) / 86400


class ProbeProbeDeflection:
	"""
	Computes the deflection maneuver delta-V and time of flight for delivering
	two atmospheric entry systems from an approach trajectory.

	Attributes
	----------
	planetObj : AMAT.planet.Planet object
		arrival planet object for approach trajectory
	probe1 : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the first entry probe
	probe2 : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the second entry probe
	r_dv_rp : float
		radial distance at which the deflection maneuver is
		performed, in terms of arrival planet radius
		Example: 1000, for maneuver performed at radial distance of
		1000 times the radius of the planet.
	r_dv : float
		radial distance at which the deflection maneuver is
		performed, in meters
	theta_star_dv_probe1 : float
		true anomaly of the first probe at deflection maneuver, rad
	theta_star_dv_probe2 : float
		true anomaly of the second probe at deflection maneuver, rad
	delta_theta_star_probe1 : float
		true anomaly change for first probe (Eq. 2.75, REF), rad
	delta_theta_star_probe2 : float
		true anomaly change for second probe (Eq. 2.75, REF), rad
	v_vec_dv_probe1 : numpy.ndarray
		first probe velocity vector in body-inertial frame, meters
	v_vec_dv_probe2 : numpy.ndarray
		second probe velocity vector in body-inertial frame, meters
	v_vec_dv_maneuver : numpy.ndarray
		deflection maneuver delta-V vector, m/s
	v_vec_dv_maneuver_mag : numpy.ndarray
		deflection maneuver delta-V magnitude, m/s
	TOF_probe1 : float
		probe time of flight from deflection maneuver until
		atmospheric entry interface, days
	TOF_probe2: float
		orbiter time of flight from deflection maneuver until
		atmospheric entry interface, days

	"""

	def __init__(self, arrivalPlanet, v_inf_vec_icrf_kms,
					   rp_probe1, psi_probe1,  h_EI_probe1,
					   rp_probe2, psi_probe2,  h_EI_probe2,
					   r_dv_rp):
		"""

		Initializes the ProbeProbeDeflection class with the following params

		Parameters
		----------
		arrivalPlanet : str
			Name of the planetary body, must be all uppercase
			Valid entries are: 'VENUS', 'EARTH', 'MARS',
			'JUPITER', 'SATURN', 'TITAN', 'URANUS', 'NEPTUNE'
		v_inf_vec_ICRF_kms : numpy.ndarray
			v_inf vector in ICRF, km/s
		rp_probe1 : float
			target periapsis radius for first entry probe, meters
		psi_probe1 : float
			target angular position on ring of periapsis radius
			for first entry probe, rad
		h_EI_probe1 : float
			atmospheric entry interface altitude for first entry
			probe, meters
		rp_probe2 : float
			target periapsis radius for second entry probe, meters
		psi_probe2 : float
			target angular position on ring of periapsis radius
			for second  entry probe, rad
		h_EI_probe2 : float
			atmospheric entry interface altitude for second entry
			probe, meters
		r_dv_rp : float
			radial distance at which the deflection manuever is
			performed, in terms of arrival planet radius
			Example: 1000, for manuever performed at radial distance of
			1000 times the radius of the planet.
		"""

		self.planetObj = Planet(arrivalPlanet)
		self.probe1 = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_probe1, psi_probe1, is_entrySystem=True, h_EI=h_EI_probe1)
		self.probe2 = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_probe2, psi_probe2, is_entrySystem=True, h_EI=h_EI_probe2)
		self.r_dv_rp = r_dv_rp
		self.r_dv = self.r_dv_rp * self.planetObj.RP

		self.theta_star_dv_probe1 = -1*np.arccos(((self.probe1.h**2/(self.planetObj.GM * self.r_dv)) - 1)*\
												(1.0/self.probe1.e))
		self.theta_star_dv_probe2 = -1 * np.arccos(((self.probe2.h ** 2 / (self.planetObj.GM * self.r_dv)) - 1) * \
												   (1.0 / self.probe2.e))

		self.r_vec_dv = self.probe1.pos_vec_bi(self.theta_star_dv_probe1)
		self.r_vec_dv_unit = self.r_vec_dv / LA.norm(self.r_vec_dv)

		self.delta_theta_star_probe1 = abs(self.theta_star_dv_probe1)
		self.delta_theta_star_probe2 = abs(self.theta_star_dv_probe2)


		self.P_probe1 = self.probe1.a*(1 - self.probe1.e**2)
		self.f_probe1 = 1 - (self.probe1.rp/self.P_probe1)*(1 - np.cos(self.delta_theta_star_probe1))
		self.g_probe1 = ((self.r_dv*self.probe1.rp)/(np.sqrt(self.planetObj.GM*self.P_probe1)))*np.sin(self.delta_theta_star_probe1)
		self.v_vec_dv_probe1 = (self.probe1.rp_vec_bi - self.f_probe1*self.r_vec_dv)/self.g_probe1

		self.P_probe2 = self.probe2.a*(1 - self.probe2.e**2)
		self.f_probe2 = 1 - (self.probe2.rp / self.P_probe2) * (1 - np.cos(self.delta_theta_star_probe2))
		self.g_probe2 = ((self.r_dv * self.probe2.rp) / (np.sqrt(self.planetObj.GM * self.P_probe2))) * np.sin(self.delta_theta_star_probe2)
		self.v_vec_dv_probe2 = (self.probe2.rp_vec_bi - self.f_probe2 * self.r_vec_dv) / self.g_probe2

		self.dv_maneuver_vec = self.v_vec_dv_probe2 - self.v_vec_dv_probe1
		self.dv_maneuver_mag = LA.norm(self.dv_maneuver_vec)

		self.H1 = np.sqrt((self.probe1.e-1)/(self.probe1.e+1))*np.tan(0.5*self.delta_theta_star_probe1)
		self.H_dv_probe1 = 2*np.arctanh(self.H1)
		self.T1 = np.sqrt(abs(self.probe1.a)**3/self.planetObj.GM)
		self.T2 = abs(self.probe1.e*np.sinh(self.H_dv_probe1) - self.H_dv_probe1)
		self.TOF_probe1 = (self.T1*self.T2) / 86400

		self.H2 = np.sqrt((self.probe2.e - 1) / (self.probe2.e + 1)) * np.tan(0.5 * self.delta_theta_star_probe2)
		self.H_dv_probe2 = 2 * np.arctanh(self.H2)
		self.T3 = np.sqrt(abs(self.probe2.a) ** 3 / self.planetObj.GM)
		self.T4 = abs(self.probe2.e * np.sinh(self.H_dv_probe2) - self.H_dv_probe2)
		self.TOF_probe2 = (self.T3 * self.T4) / 86400

class OrbiterOrbiterDeflection:
	"""
	Computes the deflection maneuver delta-V and time of flight for delivering
	two orbiter spacecraft from an approach trajectory.

	Attributes
	----------
	planetObj : AMAT.planet.Planet object
		arrival planet object for approach trajectory
	space1 : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the first orbiter spacecraft
	space2 : AMAT.approach.Approach object
		Approach object with approach trajectory parameters for the second orbiter spacecraft
	r_dv_rp : float
		radial distance at which the deflection maneuver is
		performed, in terms of arrival planet radius
		Example: 1000, for maneuver performed at radial distance of
		1000 times the radius of the planet.
	r_dv : float
		radial distance at which the deflection maneuver is
		performed, in meters
	theta_star_dv_space1 : float
		true anomaly of the first orbiter spacecraft at deflection maneuver, rad
	r_vec_dv : numpy.ndarray
		position vector at deflection maneuver in body-inertial frame, meters
	r_vec_dv_unit : numpy.ndarray
		position unit vector at deflection maneuver in body-inertial frame
	delta_theta_star_space1 : float
		true anomaly change for first orbiter spacecraft (Eq. 2.78, REF), rad
	delta_theta_star_space2 : float
		true anomaly change for second orbiter spacecraft (Eq. 2.78, REF), rad
	v_vec_dv_space1 : numpy.ndarray
		first orbiter spacecraft velocity vector in body-inertial frame, meters
	v_vec_dv_space2 : numpy.ndarray
		second orbiter spacecraft velocity vector in body-inertial frame, meters
	v_vec_dv_maneuver : numpy.ndarray
		deflection maneuver delta-V vector, m/s
	v_vec_dv_maneuver_mag : numpy.ndarray
		deflection maneuver delta-V magnitude, m/s
	TOF_space1 : float
		first spacecraft orbiter time of flight from deflection maneuver until
		periapsis, days
	TOF_space2 : float
		second spacecraft orbiter time of flight from deflection maneuver until
		periapsis, days

	"""

	def __init__(self, arrivalPlanet, v_inf_vec_icrf_kms,
								  rp_space1, psi_space1,
								  rp_space2, psi_space2,
								  r_dv_rp):
		"""

		Initializes the OrbiterOrbiterDeflection class with the following params

		Parameters
		----------
		arrivalPlanet :  str
			Name of the planetary body, must be all uppercase
			Valid entries are: 'VENUS', 'EARTH', 'MARS',
			'JUPITER', 'SATURN', 'TITAN', 'URANUS', 'NEPTUNE'
		v_inf_vec_ICRF_kms : numpy.ndarray
			v_inf vector in ICRF, km/s
		rp_space1 : float
			target periapsis radius for first orbiter spacecraft, meters
		psi_space1 : float
			target angular position on ring of periapsis radius
			for first orbiter spacecraft, rad
		rp_space2 : float
			target periapsis radius for first orbiter spacecraft, meters
		psi_space2 : float
			target angular position on ring of periapsis radius
			for first orbiter spacecraft, rad
		r_dv_rp : float
			radial distance at which the deflection manuever is
			performed, in terms of arrival planet radius
			Example: 1000, for maneuver performed at radial distance of
			1000 times the radius of the planet.

		"""

		self.planetObj = Planet(arrivalPlanet)
		self.space1 = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_space1, psi_space1)
		self.space2 = Approach(arrivalPlanet, v_inf_vec_icrf_kms, rp_space2, psi_space2)
		self.r_dv_rp = r_dv_rp
		self.r_dv = self.r_dv_rp * self.planetObj.RP

		self.theta_star_dv_space1 = -1*np.arccos(((self.space1.h**2/(self.planetObj.GM * self.r_dv)) - 1)*\
												(1.0/self.space1.e))

		self.r_vec_dv = self.space1.pos_vec_bi(self.theta_star_dv_space1)
		self.r_vec_dv_unit = self.r_vec_dv / LA.norm(self.r_vec_dv)

		self.delta_theta_star_space1 = np.arccos(np.dot(self.space1.rp_vec_bi_unit, self.r_vec_dv_unit))
		self.delta_theta_star_space2 = np.arccos(np.dot(self.space2.rp_vec_bi_unit, self.r_vec_dv_unit))


		self.C1 = np.sqrt(self.space1.rp**2 + self.r_dv**2 - 2*self.space1.rp*self.r_dv*np.cos(self.delta_theta_star_space1))
		self.S1 = 0.5*(self.space1.rp + self.r_dv + self.C1)
		self.alpha_prime1 = 2*np.arcsinh(np.sqrt(self.S1/(2*abs(self.space1.a))))
		self.beta_prime1  = 2*np.arcsinh(np.sqrt((self.S1-self.C1)/(2*abs(self.space1.a))))
		self.P1 = (4*abs(self.space1.a)*(self.S1 - self.space1.rp)*(self.S1 - self.r_dv))/(self.C1**2)
		self.P2 = np.sinh(0.5*(self.alpha_prime1 + self.beta_prime1))
		self.P_space1 = self.P1*self.P2**2
		self.f_space1 = 1 - (self.space1.rp / self.P_space1) * (1 - np.cos(self.delta_theta_star_space1))
		self.g_space1 = ((self.r_dv * self.space1.rp) / (np.sqrt(self.planetObj.GM * self.P_space1))) * np.sin(self.delta_theta_star_space1)
		self.v_vec_dv_space1 = (self.space1.rp_vec_bi - self.f_space1 * self.r_vec_dv) / self.g_space1

		self.C2 = np.sqrt(self.space2.rp ** 2 + self.r_dv ** 2 - 2 * self.space2.rp * self.r_dv * np.cos(self.delta_theta_star_space2))
		self.S2 = 0.5 * (self.space2.rp + self.r_dv + self.C2)
		self.alpha_prime2 = 2 * np.arcsinh(np.sqrt(self.S2 / (2 * abs(self.space2.a))))
		self.beta_prime2 = 2 * np.arcsinh(np.sqrt((self.S2 - self.C2) / (2 * abs(self.space2.a))))
		self.P3 = (4 * abs(self.space2.a) * (self.S2 - self.space2.rp) * (self.S2 - self.r_dv)) / (self.C2 ** 2)
		self.P4 = np.sinh(0.5 * (self.alpha_prime2 + self.beta_prime2))
		self.P_space2 = self.P3 * self.P4 ** 2
		self.f_space2 = 1 - (self.space2.rp / self.P_space2) * (1 - np.cos(self.delta_theta_star_space2))
		self.g_space2 = ((self.r_dv * self.space2.rp) / (np.sqrt(self.planetObj.GM * self.P_space2))) * np.sin(self.delta_theta_star_space2)
		self.v_vec_dv_space2 = (self.space2.rp_vec_bi - self.f_space2 * self.r_vec_dv) / self.g_space2

		self.dv_maneuver_vec = self.v_vec_dv_space2 - self.v_vec_dv_space1
		self.dv_maneuver_mag = LA.norm(self.dv_maneuver_vec)

		self.U1 = np.sqrt(abs(self.space1.a)**3/self.planetObj.GM)
		self.U2 = (np.sinh(self.alpha_prime1) - self.alpha_prime1) - (np.sinh(self.beta_prime1) - self.beta_prime1)
		self.TOF_space1 = (self.U1*self.U2)/86400

		self.U3 = np.sqrt(abs(self.space2.a) ** 3 / self.planetObj.GM)
		self.U4 = (np.sinh(self.alpha_prime2) - self.alpha_prime2) - (np.sinh(self.beta_prime2) - self.beta_prime2)
		self.TOF_space2 = (self.U3 * self.U4)/86400
